#!/usr/bin/fades

# Copyright 2018-2023 Facundo Batista <facundo@taniquetil.com.ar>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Generate X-arios from twitter feed."""

import datetime
import html
import re
import sys
from dataclasses import dataclass

import infoauth  # fades
from mastodon import Mastodon  # fades Mastodon.py

NO_TAGS = [
    (re.compile("</?span.*?>"), ""),
    (re.compile("</?a.*?>"), ""),
    (re.compile("<p>"), ""),
    (re.compile("</p>"), "\n"),
    (re.compile("<br />"), "\n"),
]


def tag_filter(txt):
    for filt, repl in NO_TAGS:
        txt = filt.sub(repl, txt)
    return txt


@dataclass
class Post:
    timestamp: datetime.date
    text: str


def get_toots(config):
    """Get info from twitter."""
    mastodon = Mastodon(access_token=config["access_token"], api_base_url=config["api_base_url"])
    own_id = mastodon.me()["id"]

    for item in mastodon.account_statuses(own_id):
        created_at = item["created_at"]
        text = html.unescape(item["content"])
        text = tag_filter(text)
        text = text.strip().replace("\n", " ∙ ")
        yield Post(created_at, text)


def main(xario_log, hashtag, auth_file):
    """Main entry point."""
    config = infoauth.load(auth_file)
    posts = get_toots(config)

    # xario
    lines = []
    real_hashtag = '#' + hashtag
    for post in posts:
        if real_hashtag not in post.text:
            continue
        clean = post.text.replace(real_hashtag, "").strip()
        tstamp = post.timestamp.strftime("%Y-%m-%d %H:%M")
        lines.append((tstamp, "{} {}\n".format(tstamp, clean)))

    # get all the timestamps in the destination file
    with open(xario_log, "rt", encoding="utf8") as fh:
        old_tstamps = {" ".join(line.split()[0:2]) for line in fh}

    # add it if new, reordering to leave latest in the end
    new_lines = [text for tstamp, text in sorted(lines) if tstamp not in old_tstamps]
    if new_lines:
        for line in new_lines:
            print("New xario line for {}: {!r}".format(hashtag, line))
        with open(xario_log, "at", encoding="utf8") as fh:
            fh.writelines(new_lines)


HELP = """
Usar xario.py xario_log hashtag auth_file
  Ej: xario.py frases.txt facundario /tmp/authfile
"""

if __name__ == "__main__":
    try:
        _, xario_log, hashtag, auth_file = sys.argv
    except ValueError:
        print(HELP)
        sys.exit()
    main(xario_log, hashtag, auth_file)
